% File lab01_ex_reconstruction.m

clear all; close all; 

% Program parameters
f0 = 1;       % frequency of signal x(n) [Hz] 
fs = 100;     % frequency of sampling [Hz] 
N = 400;      % number of signal x(n) samples 
D = 10;       % down-sampling ratio (D-times)

% Generation of sine x(n) with fx frequency
T = 1/fs;                             % sampling period for x(n)
t = 0 : T : (N-1)*T;                  % time moments for x(n)
x = sin(2*pi*f0*t);                   % generation of sine x(n)
figure; plot(t,x,'b-',t,x,'ro'); grid; title('Sine sampled with high frequency');

% Sine down-sampling: only every D-th sample is left  
xD = x(1 : D : length(x));            % down-sampled sine x(n)
ND = length(xD);  TD = D*T;  tD = 0 : TD : (ND-1)*TD; 
figure; plot(t,x,'b-',tD,xD,'ro');  grid; title('Sine sampled with low frequency');
figure; plot(tD,xD,'b-',tD,xD,'ro'); grid; title('Sine sampled with low frequency');
figure; stem(xD,'b'); title('Sine sampled with low frequency');

% Sinc(a) = sin(a)/a function
tt = -(N-1)*T : T : (N-1)*T;          % time support of Sinc function
fSinc = sin(pi/TD*tt)./(pi/TD*tt);    % Sinc function generation
fSinc(N) = 1;                         % its value for arg=0 (division 0/0) 
tz = [ -tD(end:-1:1) tD(2:ND)];       % moments of zero-crossing plus 0
z = [zeros(1,ND-1) 1 zeros(1,ND-1)];  % Sinc values at time tz
figure; plot(tt,fSinc,'b',tz,z,'o'); grid; title('Sinc function');
  
% Reconstruction of the original signal from the decimated signal
figure;
y = zeros(1,N);
for k = 1 : ND
   fSinc1 = fSinc( (N)-(k-1)*D : (2*N-1)-(k-1)*D ); 
   y1 = xD(k) * fSinc1; 
   y = y + y1; 
   subplot(311); plot(t,fSinc1); grid; title('Next shifted Sinc function'); 
   subplot(312); plot(t,y1);     grid; title('Next shifted and scaled Sinc function'); 
   subplot(313); plot(t,y);      grid; title('Summed y(t) till now'); 
 % pause
   pause(0.1);
end
figure; plot(t,y,'b',t,x,'r');     grid; title('Reconstructed signal');
figure; plot(t,x(1:N)-y(1:N),'b'); grid; title('Reconstruction error');
